import { world, system, BlockTypes, ItemStack, BlockPermutation, ItemDurabilityComponent } from '@minecraft/server';
import { Enchantments } from "classes/Enchantments.js";

const CAPITATOR_LIMIT = 164;

system.runTimeout(()=>{
    world.sendMessage(`§e[Dew's Tree Capitator] Addon Active`)
    console.warn(`[Dew's Tree Capitator] Addon Active`)
},60)

// Function to get Unbreaking level and damage chance
function getUnbreakingDetails(item) {
    let unbreakingLevel = 0;
    let damageChance = 100;
    const enchantments = Enchantments.getEnchants(item);

    for (let enchant of enchantments) {
        if (enchant.type === "unbreaking") {
            unbreakingLevel = enchant.level;
            damageChance = item.getComponent("durability").getDamageChance(unbreakingLevel);
            break;
        }
    }

    return { unbreakingLevel, damageChance };
}

// Function to update item durability
function updateItemDurability(item, currentDamage, maxDurability, damageChance) {
    const randomChance = Math.floor(Math.random() * 100) + 1;
    if (damageChance >= randomChance) {
        if (currentDamage < maxDurability) {
            return currentDamage + 1;
        } else {
            return -1;  // Indicates item breakage
        }
    }
    return currentDamage;
}

// Function to process block breaking
function processBlockBreaking(player, blockId, itemHolding, blockLocation) {
    const { damageChance } = getUnbreakingDetails(itemHolding);
    if(!itemHolding.hasComponent(ItemDurabilityComponent.componentId)) return;
    let currentDamage = itemHolding.getComponent("durability").damage;
    const maxDurability = itemHolding.getComponent("durability").maxDurability;
    let logLocations = {};
    let previousId = undefined;
    let nextBlockLoc = { ...blockLocation, y: blockLocation.y + 1 };
    for (let limit = 0; limit < CAPITATOR_LIMIT;) {
        const currentBlock = player.dimension.getBlock(nextBlockLoc);
        const locId = `${nextBlockLoc.x}:${nextBlockLoc.y}:${nextBlockLoc.z}`;
        
        if (previousId !== undefined) {
            logLocations[previousId].pass = true;
        }

        if (currentBlock.typeId.includes(blockId)) {
            previousId = locId;
            for (let y = -1; y <= 1; y++) {
                for (let x = -1; x <= 1; x++) {
                    for (let z = -1; z <= 1; z++) {
                        const neighborLocId = `${nextBlockLoc.x + x}:${nextBlockLoc.y + y}:${nextBlockLoc.z + z}`;
                        const neighborBlock = player.dimension.getBlock({ x: nextBlockLoc.x + x, y: nextBlockLoc.y + y, z: nextBlockLoc.z + z });
                        
                        if (neighborBlock.typeId.includes(blockId) && !logLocations.hasOwnProperty(neighborLocId)) {
                            limit++;
                            currentDamage = updateItemDurability(itemHolding, currentDamage, maxDurability, damageChance);

                            if (currentDamage === -1) {
                                player.playSound("random.break");
                                player.sendMessage("§c[Dew's Tree Capitator] Axe doesn't have enough durability.")
                                return;
                            }

                            logLocations[neighborLocId] = { location: { x: nextBlockLoc.x + x, y: nextBlockLoc.y + y, z: nextBlockLoc.z + z }, pass: false };
                        }
                    }
                }
            }

            nextBlockLoc = player.dimension.getBlock({ x: nextBlockLoc.x, y: nextBlockLoc.y + 1, z: nextBlockLoc.z }).typeId.includes(blockId) ?
                { x: nextBlockLoc.x, y: nextBlockLoc.y + 1, z: nextBlockLoc.z } : findNextBlock(logLocations);
        } else {
            nextBlockLoc = findNextBlock(logLocations);
        }

        if (!nextBlockLoc) break;
    }

    itemHolding.getComponent("durability").damage = currentDamage;
    player.getComponent("inventory").container.setItem(player.selectedSlotIndex, itemHolding);
    
    breakLogs(player, blockId, logLocations);
}

// Function to find the next block to process
function findNextBlock(logLocations) {
    for (let key in logLocations) {
        if (!logLocations[key].pass) {
            logLocations[key].pass = true;
            return logLocations[key].location;
        }
    }
    return null;
}

// Function to break logs and drop items
function breakLogs(player, blockId, logLocations) {
    const breakLogsInterval = system.runInterval(() => {
        const keys = Object.keys(logLocations);
        if (keys.length > 0) {
            const key = keys[0];
            const logLocation = logLocations[key].location;
            const doubleCheckBlock = player.dimension.getBlock(logLocation);

            if (blockId === doubleCheckBlock.typeId) {
                const logBlock = player.dimension.getBlock(logLocation)
                logBlock.setPermutation(BlockPermutation.resolve("minecraft:air"))
                player.dimension.spawnItem(new ItemStack(blockId, 1), logLocation);
                world.playSound("dig.wood", logLocation);
            }

            delete logLocations[key];
        } else {
            system.clearRun(breakLogsInterval);
        }
    }, 1);
}

world.afterEvents.playerBreakBlock.subscribe((eventData) => {
    const block = eventData.brokenBlockPermutation;
    const blockId = block.type.id;
    const blockLocation = eventData.block.location;
    const player = eventData.player;
    const itemHolding = player.getComponent("inventory").container.getItem(player.selectedSlotIndex);
    
    if (!itemHolding) return;

    const isAxe = itemHolding.typeId.includes("_axe") && itemHolding.typeId.includes("minecraft:");
    const isLog = blockId.includes("log");

    if (player.isSneaking && isAxe && isLog) {
        const itemHoldingDurability = itemHolding.getComponent(ItemDurabilityComponent.componentId).damage
        
        processBlockBreaking(player, blockId, itemHolding, blockLocation);
    }
});
